namespace BookCatalog.Areas.Admin.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using BookCatalog.Models;
    using BookCatalog.Models.Repositories;
   
    public class CatalogController : Controller
    {
        private readonly ILanguageRepository _languageRepository;
        private readonly IPublisherRepository _publisherRepository;
        private readonly IBookDetailsRepository _bookdetailsRepository;

        // If you are using Dependency Injection, you can delete the following constructor
        public CatalogController()
            : this(new LanguageRepository(), new PublisherRepository(), new BookDetailsRepository())
        {
        }

        public CatalogController(
            ILanguageRepository languageRepository, 
            IPublisherRepository publisherRepository, 
            IBookDetailsRepository bookdetailsRepository)
        {
            this._languageRepository = languageRepository;
            this._publisherRepository = publisherRepository;
            this._bookdetailsRepository = bookdetailsRepository;
        }

        // GET: /Catalog/
        public ViewResult Index()
        {
            return this.View(
                this._bookdetailsRepository.AllIncluding(
                    bookdetails => bookdetails.Language, 
                    bookdetails => bookdetails.Publisher,
                    bookdetails => bookdetails.Tags));
        }

        // GET: /Catalog/Create
        public ActionResult Create()
        {
            this.ViewBag.PossibleLanguages = this._languageRepository.All;
            this.ViewBag.PossiblePublishers = this._publisherRepository.All;
            return this.View();
        } 

        // POST: /Catalog/Create
        [HttpPost]
        public ActionResult Create(BookDetails bookdetails)
        {
            if (ModelState.IsValid) {
                this._bookdetailsRepository.InsertOrUpdate(bookdetails);
                this._bookdetailsRepository.Save();
                return this.RedirectToAction("Index");
            } 
            
            this.ViewBag.PossibleLanguages = this._languageRepository.All;
            this.ViewBag.PossiblePublishers = this._publisherRepository.All;
            return this.View();
        }
        
        // GET: /Catalog/Edit/{id}
        public ActionResult Edit(int id)
        {
            this.ViewBag.PossibleLanguages = this._languageRepository.All;
            this.ViewBag.PossiblePublishers = this._publisherRepository.All;
            return this.View(this._bookdetailsRepository.Find(id));
        }

        // POST: /Catalog/Edit/{id}
        [HttpPost]
        public ActionResult Edit(BookDetails bookdetails)
        {
            if (ModelState.IsValid) {
                this._bookdetailsRepository.InsertOrUpdate(bookdetails);
                this._bookdetailsRepository.Save();
                return this.RedirectToAction("Index");
            } 

            this.ViewBag.PossibleLanguages = this._languageRepository.All;
            this.ViewBag.PossiblePublishers = this._publisherRepository.All;
            return this.View();
        }

        // GET: /Catalog/Delete/{id}
        public ActionResult Delete(int id)
        {
            return this.View(this._bookdetailsRepository.Find(id));
        }

        // POST: /Catalog/Delete/{id}
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            this._bookdetailsRepository.Delete(id);
            this._bookdetailsRepository.Save();

            return this.RedirectToAction("Index");
        }
    }
}