namespace BookCatalog.Areas.Admin.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using BookCatalog.Models;
    using BookCatalog.Models.Repositories;
   
    public class LanguagesController : Controller
    {
        private readonly ILanguageRepository _languageRepository;

        // If you are using Dependency Injection, you can delete the following constructor
        public LanguagesController()
            : this(new LanguageRepository())
        {
        }

        public LanguagesController(ILanguageRepository languageRepository)
        {
            this._languageRepository = languageRepository;
        }

        // GET: /Languages/
        public ViewResult Index()
        {
            return this.View(this._languageRepository.All);
        }

        // GET: /Languages/Create
        public ActionResult Create()
        {
            return this.View();
        } 

        // POST: /Languages/Create
        [HttpPost]
        public ActionResult Create(Language language)
        {
            if (ModelState.IsValid) {
                this._languageRepository.InsertOrUpdate(language);
                this._languageRepository.Save();
                return this.RedirectToAction("Index");
            } 
            
            return this.View();
        }
        
        // GET: /Languages/Edit/{id}
        public ActionResult Edit(int id)
        {
            return this.View(this._languageRepository.Find(id));
        }

        // POST: /Languages/Edit/{id}
        [HttpPost]
        public ActionResult Edit(Language language)
        {
            if (ModelState.IsValid) {
                this._languageRepository.InsertOrUpdate(language);
                this._languageRepository.Save();
                return this.RedirectToAction("Index");
            } 

            return this.View();
        }

        // GET: /Languages/Delete/{id}
        public ActionResult Delete(int id)
        {
            return this.View(this._languageRepository.Find(id));
        }

        // POST: /Languages/Delete/{id}
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            this._languageRepository.Delete(id);
            this._languageRepository.Save();

            return this.RedirectToAction("Index");
        }
    }
}