namespace BookCatalog.Areas.Admin.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using BookCatalog.Models;
    using BookCatalog.Models.Repositories;
   
    public class PublishersController : Controller
    {
        private readonly IPublisherRepository _publisherRepository;

        // If you are using Dependency Injection, you can delete the following constructor
        public PublishersController()
            : this(new PublisherRepository())
        {
        }

        public PublishersController(IPublisherRepository publisherRepository)
        {
            this._publisherRepository = publisherRepository;
        }

        // GET: /Publishers/
        public ViewResult Index()
        {
            return this.View(
                this._publisherRepository.AllIncluding(publisher => publisher.Books));
        }

        // GET: /Publishers/Create
        public ActionResult Create()
        {
            return this.View();
        } 

        // POST: /Publishers/Create
        [HttpPost]
        public ActionResult Create(Publisher publisher)
        {
            if (ModelState.IsValid) {
                this._publisherRepository.InsertOrUpdate(publisher);
                this._publisherRepository.Save();
                return this.RedirectToAction("Index");
            } 
            
            return this.View();
        }
        
        // GET: /Publishers/Edit/{id}
        public ActionResult Edit(int id)
        {
            return this.View(this._publisherRepository.Find(id));
        }

        // POST: /Publishers/Edit/{id}
        [HttpPost]
        public ActionResult Edit(Publisher publisher)
        {
            if (ModelState.IsValid) {
                this._publisherRepository.InsertOrUpdate(publisher);
                this._publisherRepository.Save();
                return this.RedirectToAction("Index");
            } 

            return this.View();
        }

        // GET: /Publishers/Delete/{id}
        public ActionResult Delete(int id)
        {
            return this.View(this._publisherRepository.Find(id));
        }

        // POST: /Publishers/Delete/{id}
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            this._publisherRepository.Delete(id);
            this._publisherRepository.Save();

            return this.RedirectToAction("Index");
        }
    }
}