namespace BookCatalog.Areas.Admin.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using BookCatalog.Models;
    using BookCatalog.Models.Repositories;
   
    public class TagsController : Controller
    {
        private readonly ITagRepository _tagRepository;

        // If you are using Dependency Injection, you can delete the following constructor
        public TagsController()
            : this(new TagRepository())
        {
        }

        public TagsController(ITagRepository tagRepository)
        {
            this._tagRepository = tagRepository;
        }

        // GET: /Tags/
        public ViewResult Index()
        {
            return this.View(this._tagRepository.AllIncluding(tag => tag.Books));
        }

        // GET: /Tags/Create
        public ActionResult Create()
        {
            return this.View();
        } 

        // POST: /Tags/Create
        [HttpPost]
        public ActionResult Create(Tag tag)
        {
            if (ModelState.IsValid) {
                this._tagRepository.InsertOrUpdate(tag);
                this._tagRepository.Save();
                return this.RedirectToAction("Index");
            } 
            
            return this.View();
        }
        
        // GET: /Tags/Edit/{id}
        public ActionResult Edit(int id)
        {
            return this.View(this._tagRepository.Find(id));
        }

        // POST: /Tags/Edit/{id}
        [HttpPost]
        public ActionResult Edit(Tag tag)
        {
            if (ModelState.IsValid) {
                this._tagRepository.InsertOrUpdate(tag);
                this._tagRepository.Save();
                return this.RedirectToAction("Index");
            } 

            return this.View();
        }

        // GET: /Tags/Delete/{id}
        public ActionResult Delete(int id)
        {
            return this.View(this._tagRepository.Find(id));
        }

        // POST: /Tags/Delete/{id}
        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            this._tagRepository.Delete(id);
            this._tagRepository.Save();

            return this.RedirectToAction("Index");
        }
    }
}