﻿namespace BookCatalog.Models.DbContext
{
    using System;
    using System.Data.Entity;

    public class CatalogInitializer : DropCreateDatabaseIfModelChanges<CatalogContext>
    {
        protected override void Seed(CatalogContext context)
        {
            base.Seed(context);

            var englishLang = new Language() { Name = "English" };
            context.Languages.Add(englishLang);

            var russianLang = new Language() { Name = "Russian" };
            context.Languages.Add(russianLang);

            var microsoftPress = new Publisher() {
                Title = "Microsoft Press",
                Homepage = @"http://www.microsoft.com/mspress/"
            };
            context.Publishers.Add(microsoftPress);

            var tagVS2010 = new Tag() { Text = "VS2010" };
            context.Tags.Add(tagVS2010);

            var tagASPNET = new Tag() { Text = "ASP.NET" };
            context.Tags.Add(tagASPNET);

            var tagMVC = new Tag() { Text = "MVC" };
            context.Tags.Add(tagMVC);

            var tagCSharp = new Tag() { Text = "C#" };
            context.Tags.Add(tagCSharp);

            var tagWP7 = new Tag() { Text = "WP7" };
            context.Tags.Add(tagWP7);

            context.Books.Add(new BookDetails() {
                Title = "ASP.NET Web Pages with Razor Syntax",
                Author = "P. Pelland, P. Pare, Ken Haines",
                Language = englishLang,
                Publisher = microsoftPress,
                PublishedAt = new DateTime(2010, 09, 10),
                Url = @"http://blogs.msdn.com/b/microsoft_press/archive/2010/09/13/free-ebook-moving-to-microsoft-visual-studio-2010.aspx",
                Description = "For developers moving from Visual Studio 2003,2005,2008 to Visual Studio 2010.",
                Rating = 4,
                Tags = new Tag[1] { tagVS2010 },
                IsFree = true,
                IsVisible = true
            });

            context.Books.Add(new BookDetails() {
                Title = "Programming Windows Phone 7",
                Author = "Charles Petzold",
                Language = englishLang,
                Publisher = microsoftPress,
                PublishedAt = new DateTime(2010, 10, 28),
                Url = @"http://www.charlespetzold.com/phone/",
                Description = "This book is a gift from the Windows Phone 7 team at Microsoft to the programming community.",
                Rating = 5,
                Tags = new Tag[3] { tagVS2010, tagCSharp, tagWP7 },
                IsFree = true,
                IsVisible = true
            });
        }
    }
}