namespace BookCatalog.Models.Repositories
{ 
    using System;
    using System.Data;
    using System.Data.Entity;
    using System.Linq;
    using System.Linq.Expressions;
    using BookCatalog.Models;
    using BookCatalog.Models.DbContext;
    
    public interface IBookDetailsRepository
    {
        IQueryable<BookDetails> All { get; }
        IQueryable<BookDetails> AllIncluding(params Expression<Func<BookDetails, object>>[] includeProperties);
        BookDetails Find(int id);
        void InsertOrUpdate(BookDetails bookdetails);
        void Delete(int id);
        void Save();
    }

    public class BookDetailsRepository : IBookDetailsRepository
    {
        private readonly CatalogContext _context = new CatalogContext();

        public IQueryable<BookDetails> All
        {
            get { return this._context.Books; }
        }

        public IQueryable<BookDetails> AllIncluding(params Expression<Func<BookDetails, object>>[] includeProperties)
        {
            IQueryable<BookDetails> query = this._context.Books;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }

            return query;
        }

        public BookDetails Find(int id)
        {
            return this._context.Books.Find(id);
        }

        public void InsertOrUpdate(BookDetails bookdetails)
        {
            if (bookdetails.Id == default(int)) {
                // New entity
                this._context.Books.Add(bookdetails);
            } else {
                // Existing entity
                this._context.Entry(bookdetails).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var bookdetails = this._context.Books.Find(id);
            this._context.Books.Remove(bookdetails);
        }

        public void Save()
        {
            this._context.SaveChanges();
        }
    }
}