namespace BookCatalog.Models.Repositories
{ 
    using System;
    using System.Data;
    using System.Data.Entity;
    using System.Linq;
    using System.Linq.Expressions;
    using BookCatalog.Models;
    using BookCatalog.Models.DbContext;
    
    public interface ILanguageRepository
    {
        IQueryable<Language> All { get; }
        IQueryable<Language> AllIncluding(params Expression<Func<Language, object>>[] includeProperties);
        Language Find(int id);
        void InsertOrUpdate(Language language);
        void Delete(int id);
        void Save();
    }

    public class LanguageRepository : ILanguageRepository
    {
        private readonly CatalogContext _context = new CatalogContext();

        public IQueryable<Language> All
        {
            get { return this._context.Languages; }
        }

        public IQueryable<Language> AllIncluding(params Expression<Func<Language, object>>[] includeProperties)
        {
            IQueryable<Language> query = this._context.Languages;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }

            return query;
        }

        public Language Find(int id)
        {
            return this._context.Languages.Find(id);
        }

        public void InsertOrUpdate(Language language)
        {
            if (language.Id == default(int)) {
                // New entity
                this._context.Languages.Add(language);
            } else {
                // Existing entity
                this._context.Entry(language).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var language = this._context.Languages.Find(id);
            this._context.Languages.Remove(language);
        }

        public void Save()
        {
            this._context.SaveChanges();
        }
    }
}