namespace BookCatalog.Models.Repositories
{ 
    using System;
    using System.Data;
    using System.Data.Entity;
    using System.Linq;
    using System.Linq.Expressions;
    using BookCatalog.Models;
    using BookCatalog.Models.DbContext;
    
    public interface IPublisherRepository
    {
        IQueryable<Publisher> All { get; }
        IQueryable<Publisher> AllIncluding(params Expression<Func<Publisher, object>>[] includeProperties);
        Publisher Find(int id);
        void InsertOrUpdate(Publisher publisher);
        void Delete(int id);
        void Save();
    }

    public class PublisherRepository : IPublisherRepository
    {
        private readonly CatalogContext _context = new CatalogContext();

        public IQueryable<Publisher> All
        {
            get { return this._context.Publishers; }
        }

        public IQueryable<Publisher> AllIncluding(params Expression<Func<Publisher, object>>[] includeProperties)
        {
            IQueryable<Publisher> query = this._context.Publishers;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }

            return query;
        }

        public Publisher Find(int id)
        {
            return this._context.Publishers.Find(id);
        }

        public void InsertOrUpdate(Publisher publisher)
        {
            if (publisher.Id == default(int)) {
                // New entity
                this._context.Publishers.Add(publisher);
            } else {
                // Existing entity
                this._context.Entry(publisher).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var publisher = this._context.Publishers.Find(id);
            this._context.Publishers.Remove(publisher);
        }

        public void Save()
        {
            this._context.SaveChanges();
        }
    }
}