namespace BookCatalog.Models.Repositories
{ 
    using System;
    using System.Data;
    using System.Data.Entity;
    using System.Linq;
    using System.Linq.Expressions;
    using BookCatalog.Models;
    using BookCatalog.Models.DbContext;
    
    public interface ITagRepository
    {
        IQueryable<Tag> All { get; }
        IQueryable<Tag> AllIncluding(params Expression<Func<Tag, object>>[] includeProperties);
        Tag Find(int id);
        void InsertOrUpdate(Tag tag);
        void Delete(int id);
        void Save();
    }

    public class TagRepository : ITagRepository
    {
        private readonly CatalogContext _context = new CatalogContext();

        public IQueryable<Tag> All
        {
            get { return this._context.Tags; }
        }

        public IQueryable<Tag> AllIncluding(params Expression<Func<Tag, object>>[] includeProperties)
        {
            IQueryable<Tag> query = this._context.Tags;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }

            return query;
        }

        public Tag Find(int id)
        {
            return this._context.Tags.Find(id);
        }

        public void InsertOrUpdate(Tag tag)
        {
            if (tag.Id == default(int)) {
                // New entity
                this._context.Tags.Add(tag);
            } else {
                // Existing entity
                this._context.Entry(tag).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var tag = this._context.Tags.Find(id);
            this._context.Tags.Remove(tag);
        }

        public void Save()
        {
            this._context.SaveChanges();
        }
    }
}